<?php

namespace Pinpoint\KlarnaCapture\Observer;

use Exception;
use Klarna\Kco\Model\Payment\Kco;
use Klarna\Kp\Model\Payment\Kp;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\DB\Transaction;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Message\Manager as MessageManager;
use Magento\Sales\Api\InvoiceRepositoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Service\InvoiceService;
use Pinpoint\KlarnaCapture\Helper\Data;
use Psr\Log\LoggerInterface;

/**
 * Capture on order place.
 */
class KlarnaCapture implements ObserverInterface
{
    /**
     * @var MessageManager
     */
    protected $messageManager;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var AbstractHelper
     */
    protected $helper;

    /**
     * @var InvoiceService
     */
    protected $invoiceService;

    /**
     * @var Transaction
     */
    protected $transaction;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var InvoiceRepositoryInterface
     */
    protected $invoiceRepository;

    /**
     * @param MessageManager             $messageManager
     * @param LoggerInterface            $logger
     * @param Data                       $helper
     * @param InvoiceService             $invoiceService
     * @param Transaction                $transaction
     * @param OrderRepositoryInterface   $orderRepository
     * @param InvoiceRepositoryInterface $invoiceRepository
     */
    public function __construct(
        MessageManager $messageManager,
        LoggerInterface $logger,
        Data $helper,
        InvoiceService $invoiceService,
        Transaction $transaction,
        OrderRepositoryInterface $orderRepository,
        InvoiceRepositoryInterface $invoiceRepository
    ) {
        $this->messageManager = $messageManager;
        $this->logger = $logger;
        $this->helper = $helper;
        $this->invoiceService = $invoiceService;
        $this->transaction = $transaction;
        $this->orderRepository = $orderRepository;
        $this->invoiceRepository = $invoiceRepository;
    }

    /**
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /** @var Order $order */
        $order = $observer->getData('order');
        $payment = $order->getPayment();
        if (!$this->helper->getIsEnabled() ||
            !$payment ||
            !$this->isPaidUsingKlarna($payment->getMethod()) ||
            !$order->canInvoice()) {
            return;
        }
        try {
            $invoice = $this->invoiceService->prepareInvoice($order);
            $invoice->register();
            $invoice->capture();
            $transaction = $this->transaction->addObject($invoice)->addObject($invoice->getOrder());
            $transaction->save();
        } catch (Exception $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
            $this->logger->critical(__('Could not capture order #' . $order->getIncrementId()));
            $this->logger->critical($e->getMessage());
        }
    }

    /**
     * Check if this is paid using Klarna.
     *
     * @param string $method
     * @return boolean
     */
    protected function isPaidUsingKlarna(string $method): bool
    {
        return in_array($method, [
            Kco::METHOD_CODE,
            Kp::METHOD_CODE
        ]);
    }
}

